'use strict';

exports.__esModule = true;
exports.title_position = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _mobx = require('mobx');

var _FormModel = require('./FormModel');

var _utils = require('../utils');

var _FormEditModel = require('./FormEditModel');

var _FormEditModel2 = _interopRequireDefault(_FormEditModel);

var _RelationModel = require('./RelationModel');

var _RelationModel2 = _interopRequireDefault(_RelationModel);

var _FieldModel = require('./FieldModel');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var title_position = exports.title_position = {
    TOP: 'top',
    LEFT: 'left',
    RIGHT: 'right'
};

var cell_nodes = {};

var FieldDesignModel = function (_BaseFormModel) {
    _inherits(FieldDesignModel, _BaseFormModel);

    function FieldDesignModel() {
        _classCallCheck(this, FieldDesignModel);

        var _this = _possibleConstructorReturn(this, _BaseFormModel.call(this));

        (0, _mobx.extendObservable)(_this, {
            formName: '', // 表单名称
            selectKey: '', // 拖动是选择的布局id
            mainElement: null, // 布局主窗口的Element
            fieldsCollection: [], // 问题集合
            headerStyle: {
                fontSize: '18px',
                textAlign: 'center',
                fontWeight: 400,
                color: '#333'
            },
            border: {
                color: '#F0F0F0',
                show: true
            },
            is_static: false,
            layouts: [// 布局集合
            _this.getlayoutItem()],
            get fields() {
                return this.fieldsCollection.filter(function (item) {
                    return item.field_type !== _FormEditModel.field_types.LABEL;
                });
            },
            /**
             * @name 初始化
             * @param name 标题的名称
             * @param headerStyle 表头样式
             */
            init: (0, _mobx.action)(function (_ref) {
                var name = _ref.name,
                    headerStyle = _ref.headerStyle;

                //this.design = [ ];
                _this.formName = name;
                if (headerStyle) {
                    _this.headerStyle = Object.assign({}, _this.headerStyle, headerStyle);
                }
            }),
            /**
             * @name 更改标题
             * @param name 标题
             */
            changeFormName: (0, _mobx.action)(function (name) {
                _this.change = true;
                _this.formName = name;
            }),
            /**
             * @name 更改标题样式
             */
            changeHeaderStyle: (0, _mobx.action)(function (style) {
                _this.change = true;
                _this.headerStyle = Object.assign({}, _this.headerStyle, style);
            }),
            changeBorder: (0, _mobx.action)(function (border) {
                _this.border = Object.assign({}, _this.border, border);
            }),
            /**
             * @name 设置主窗口的Element
             */
            setMainElement: (0, _mobx.action)(function (_el) {
                _this.mainElement = _el;
            }),
            /**
             * @name 通过field_id获取field的store和view
             * @param id field_id
             */
            getField: function getField(id) {
                var _store = _this.fieldsCollection.filter(function (item) {
                    return item.id === id;
                });
                return {
                    store: _store[0],
                    view: _FieldModel.mapType.getView(_store[0].field_type)
                };
            },
            /**
             * @name 布局更改时更新layouts
             * @param layouts:[{i,x,y,w,h}]
             */
            layoutsChange: (0, _mobx.action)(function (layouts) {
                var _layouts = (0, _utils.copy)(_this.layouts);

                var _loop = function _loop(i) {
                    var _index = _layouts.findIndex(function (item) {
                        return item.key === layouts[i].i;
                    });
                    _layouts[_index].layout = {
                        x: layouts[i].x,
                        y: layouts[i].y,
                        w: layouts[i].w,
                        h: layouts[i].h
                    };
                    if (cell_nodes[layouts[i].i]) {
                        _layouts[_index].area = _this.getCellPosition(cell_nodes[layouts[i].i]);
                    }
                };

                for (var i = 0; i < layouts.length; i++) {
                    _loop(i);
                };
                _this.layouts = _layouts;
            }),
            /**
             * @name 添加一个布局元素
             */
            addCell: (0, _mobx.action)(function () {
                var _y = 0,
                    _h = 8;
                var _layouts = (0, _utils.copy)(_this.layouts);
                for (var i = 0; i < _layouts.length; i++) {
                    var _layouts$i$layout = _layouts[i].layout,
                        y = _layouts$i$layout.y,
                        h = _layouts$i$layout.h;

                    if (y > _y) {
                        _y = y;
                        _h = h;
                    };
                };
                _layouts.push(_this.getlayoutItem({
                    layout: { x: 0, y: _y + _h, w: 12, h: 6 }
                }));
                _this.layouts = _layouts;
            }),
            /**
             * @name 删除一个布局元素
             * @param key 选择元素的唯一标识
             */
            deleteCell: (0, _mobx.action)(function (key) {
                var _layouts = (0, _utils.copy)(_this.layouts);
                var _index = _layouts.findIndex(function (item) {
                    return item.key === key;
                });
                _layouts.splice(_index, 1);
                _this.layouts = _layouts;
                delete cell_nodes[key];
            }),
            /**
             * @name 储存布局元素的DOM节点
             * @param key 布局元素的唯一标识
             * @param node 布局元素的节点
             */
            setCellNode: function setCellNode(key, node) {
                cell_nodes[key] = node.offsetParent;
                _this.setCellPosition(key);
            },

            changeCellContentVertical: (0, _mobx.action)(function (key, value) {
                var _layouts = (0, _utils.copy)(_this.layouts);
                var _index = _layouts.findIndex(function (item) {
                    return item.key === key;
                });
                _layouts[_index].cellVertical = value;
                _this.layouts = _layouts;
            }),
            /**
             * @name 设置布局元素的有效区域（当有field拖动时通过该区域计算是否在次元素内）
             * @param key 布局元素的唯一标识
             */
            setCellPosition: (0, _mobx.action)(function (key) {
                var _layouts = (0, _utils.copy)(_this.layouts);
                var _index = _layouts.findIndex(function (item) {
                    return item.key === key;
                });
                _layouts[_index].area = _this.getCellPosition(cell_nodes[key]);
                _this.layouts = _layouts;
            }),
            /**
             * @name 获取布局元素节点在DOM内的坐标
             * @param node 布局元素的DOM节点
             */
            getCellPosition: (0, _mobx.action)(function (node) {
                var _offset = eval(node.style.transform.replace(/translate/, '').replace(/px/g, '').replace(/\(/, '[').replace(/\)/, ']'));
                var _pos = {
                    x: 0,
                    y: 0
                };
                if (_offset) {
                    _pos.x = _offset[0], _pos.y = _offset[1];
                };
                _FormEditModel2.default.getOffeset(_pos, node);
                var _width = node.clientWidth;
                var _Height = node.clientHeight;
                var _area = {
                    x: {
                        min: _pos.x,
                        max: _pos.x + _width
                    },
                    y: {
                        min: _pos.y,
                        max: _pos.y + _Height
                    }
                };
                return _area;
            }),
            /**
             * @name 当有field拖动时确定在哪个布局元素内
             * @param l 拖动元素的横坐标
             * @param t 拖动元素的纵坐标
             */
            changeElementInit: (0, _mobx.action)(function (l, t) {
                var _layouts = (0, _utils.copy)(_this.layouts);
                _this.selectKey = '';
                var _top = _this.mainElement.scrollTop;
                var _left = _this.mainElement.scrollLeft;
                var backgroundColor = 'rgb(255, 255, 255)';
                for (var i = 0; i < _layouts.length; i++) {
                    var _layouts$i$area = _layouts[i].area,
                        x = _layouts$i$area.x,
                        y = _layouts$i$area.y;

                    if (l <= x.max - _left && l >= x.min - _left && t <= y.max - _top && t >= y.min - _top) {
                        _this.selectKey = _layouts[i].key;
                        backgroundColor = '#F9F9F9';
                    } else {
                        backgroundColor = 'rgb(255, 255, 255)';
                    };
                    _layouts[i].style = Object.assign({}, _layouts[i].style, { backgroundColor: backgroundColor });
                };
                _this.layouts = _layouts;
            }),
            /**
             * @name 清除所有布局元素的选中状态
             */
            setElementInitCancel: (0, _mobx.action)(function () {
                var _layouts = (0, _utils.copy)(_this.layouts);
                for (var i = 0; i < _layouts.length; i++) {
                    _layouts[i].style = Object.assign({}, _layouts[i].style, { backgroundColor: 'rgb(255,255,255)' });
                };
                _this.layouts = _layouts;
            }),
            /**
             * @name 在fieldCollection添加一道问题-*
             */
            addField: (0, _mobx.action)(function (type) {
                if (_this.selectKey === '') {
                    return void 0;
                };
                var _store = _FieldModel.mapType.getStore(type);
                _store.setFieldOption({ id: new Date().valueOf().toString() });
                var _fields = (0, _utils.copy)(_this.fieldsCollection);
                _fields.push(_store);
                _this.fieldsCollection = _fields;
                var _layouts = (0, _utils.copy)(_this.layouts);
                var _index = _layouts.findIndex(function (item) {
                    return item.key === _this.selectKey;
                });
                _layouts[_index].field_id = _store.id;
                _this.layouts = _layouts;
            }),
            changeFieldCollection: (0, _mobx.action)(function (id, value) {
                var _collection = (0, _utils.copy)(_this.fieldsCollection);
                var _index = _collection.findIndex(function (item) {
                    return item.id === id;
                });
                _collection[_index].setFieldOption(value);
                _this.fieldsCollection = _collection;
            }),
            clearFieldValue: (0, _mobx.action)(function () {
                var _fields = (0, _utils.copy)(_this.fieldsCollection);
                for (var i = 0; i < _fields.length; i++) {
                    _fields[i].clearValue();
                };
                _this.fieldsCollection = _fields;
            }),
            addRelation: (0, _mobx.action)(function (id) {
                if (_RelationModel2.default.ctrl && !(0, _utils.isEmpty)(id)) {
                    _this.is_static = true;
                    //crfRelation.changeShowCanvas();
                    console.log(id);
                };
            })
        });
        return _this;
    }
    /**
     * @name 获取一个布局元素的初始模板
     * @param {*} options 
     */


    FieldDesignModel.prototype.getlayoutItem = function getlayoutItem(options) {
        return Object.assign({
            key: new Date().valueOf().toString(),
            field_id: '',
            cellVertical: 'top',
            style: {
                backgroundColor: 'rgba(255, 255, 255,1)'
            },
            layout: { x: 0, y: 0, w: 12, h: 8 }
        }, options);
    };

    FieldDesignModel.prototype.fieldName = function fieldName(id) {
        var _field = this.fields.filter(function (item) {
            return item.id === id;
        });
        if (_field.length > 0) {
            return _field[0].label;
        };
        return '未定义';
    };

    _createClass(FieldDesignModel, [{
        key: 'fontSizeOptions',
        get: function get() {
            var _arr = [];
            for (var i = 12; i < 64; i++) {
                _arr.push(i);
            };
            return _arr;
        }
    }, {
        key: 'fontWeightOptions',
        get: function get() {
            return ['normal', '400', '500', '600', 'bold'];
        }
    }, {
        key: 'textAlignOptions',
        get: function get() {
            return [{
                value: 'left',
                label: '左'
            }, {
                value: 'center',
                label: '中'
            }, {
                value: 'right',
                label: '右'
            }];
        }
    }]);

    return FieldDesignModel;
}(_FormModel.BaseFormModel);

;

var crfDesign = new FieldDesignModel();

exports.default = crfDesign;