'use strict';

exports.__esModule = true;
exports.mapType = exports.CRFRadioModel = exports.CRFAutoCompleteModel = exports.CRFTextModel = undefined;

var _mobx = require('mobx');

var _utils = require('../utils');

var _FormEditModel = require('./FormEditModel');

var _app = require('../app');

var _app2 = _interopRequireDefault(_app);

var _Validate = require('../Validate');

var _Validate2 = _interopRequireDefault(_Validate);

var _FieldView = require('./FieldView');

var _FieldSetting = require('./FieldSetting');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var edit_title = {
    BASE: {
        type: '_base',
        label: '基础信息',
        icon: 'icon-rulers'
    },
    OPTIONS: {
        type: '_options',
        label: '属性设置',
        icon: 'icon-cog6'
    },
    VAILDATE: {
        type: '_vaildate',
        label: '数据验证',
        icon: 'icon-clipboard2'
    },
    DEPEND: {
        type: '_depend',
        label: '逻辑关系',
        icon: 'icon-git-pull-request'
    }
};

var settings = {
    TITLE: _FieldSetting.TitleView,
    PLCEHOLDER: _FieldSetting.PlaceHolderView,
    CODE: _FieldSetting.CodeView,
    SHOWLABEL: _FieldSetting.ShowLabelView,
    REQUIRED: _FieldSetting.RequiredView,
    DESCRIPTION: _FieldSetting.DescriptionView,
    INLINE: _FieldSetting.InlineView,
    OPTIONSINLINE: _FieldSetting.OptionsInlineView,
    SETOPTIONS: _FieldSetting.SetOptionsView,
    VALIDATECODE: _FieldSetting.ValidateView
};

var getFieldOptions = function getFieldOptions() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return Object.assign({}, {
        description: '' //备注信息
        // code_validators: [{
        //     value: '',
        //     message: ''
        // }] //逻辑验证
    }, options);
};

var getValidate = function getValidate() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    return [{
        val: _Validate2.default.require,
        message: '必填项不能为空'
    }].concat(options);
};

var getSettings = function getSettings() {
    var _Object$assign;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return Object.assign({}, (_Object$assign = {}, _Object$assign[edit_title.BASE.type] = [settings.CODE, settings.TITLE, settings.PLCEHOLDER, settings.SHOWLABEL, settings.INLINE, settings.DESCRIPTION], _Object$assign[edit_title.VAILDATE.type] = [settings.REQUIRED, settings.VALIDATECODE], _Object$assign[edit_title.DEPEND.type] = [], _Object$assign), options);
};

var CRFFieldBaseModel = function CRFFieldBaseModel(options) {
    var _this = this;

    _classCallCheck(this, CRFFieldBaseModel);

    var _default = {
        id: 0,
        code: '0',
        get cid() {
            return 'c' + this.id;
        },
        setting_selected: edit_title.BASE.type,
        setting_title: [edit_title.BASE, edit_title.VAILDATE, edit_title.DEPEND],
        settings: getSettings(),
        label: '标题', // 问题标题
        field_options: getFieldOptions(),
        validates: [],
        value: '',
        hasError: false,
        errorMsg: '',
        disabled: false,
        showLabel: true, // 是否显示标题
        inline: false, // 标题与输入是否水平排列
        field_type: _FormEditModel.field_types.TEXT, // 为题类型
        required: false, // 是否必填
        get code_validators() {
            return this.validates.filter(function (item) {
                return item.type === 'error';
            });
        },
        get code2_validators() {
            return this.validates.filter(function (item) {
                return item.type === 'tip';
            });
        },
        addValidate: function addValidate(type) {
            var _validates = (0, _utils.copy)(_this.validates);
            _validates.push({
                value: '',
                message: type + ' message',
                type: type
            });
            return _validates;
        },
        changeValidates: function changeValidates(index, name, value) {
            var _validates = (0, _utils.copy)(_this.validates);
            _validates[index][name] = value;
            return _validates;
        },
        deleteValidates: function deleteValidates(index) {
            var _validates = (0, _utils.copy)(_this.validates);
            _validates.splice(index, 1);
            return _validates;
        },
        changeValue: (0, _mobx.action)(function (value) {
            _this.value = value;
            _this.hasError = false;
            for (var i = 0; i < _this.validates.length; i++) {
                if (!_this.validates[i].val(value)) {
                    _this.hasError = true;
                    errorMsg = _this.validates[i].message;
                };
            };
        }),
        setFieldOption: (0, _mobx.action)(function (_field) {
            //console.log( _field )
            for (var key in _field) {
                var _fieldArr = key.split('.');
                if (_fieldArr.length > 1) {
                    var _key = _fieldArr.splice(0, 1);
                    _this[_key] = (0, _utils.setByName)(_this[_key], _fieldArr.join('.'), _field[key]);
                    continue;
                };
                _this[key] = _field[key];
            };
        }),
        clearValue: (0, _mobx.action)(function () {
            _this.value = '';
        }),
        getOptions: function getOptions(name) {
            return (0, _utils.getValueByDeepName)(_this, name);
        }
    };
    if ((0, _utils.isObject)(options)) {
        _default = Object.assign({}, _default, options);
    };

    (0, _mobx.extendObservable)(this, _default);
};

;

var CRFTextModel = exports.CRFTextModel = function (_CRFFieldBaseModel) {
    _inherits(CRFTextModel, _CRFFieldBaseModel);

    function CRFTextModel(options) {
        _classCallCheck(this, CRFTextModel);

        return _possibleConstructorReturn(this, _CRFFieldBaseModel.call(this, {
            placeholder: '' // 为空提示
        }));
    }

    return CRFTextModel;
}(CRFFieldBaseModel);

;

var CRFAutoCompleteModel = exports.CRFAutoCompleteModel = function (_CRFFieldBaseModel2) {
    _inherits(CRFAutoCompleteModel, _CRFFieldBaseModel2);

    function CRFAutoCompleteModel(options) {
        var _this4 = this;

        var _this3;

        _classCallCheck(this, CRFAutoCompleteModel);

        return _this3 = _possibleConstructorReturn(this, _CRFFieldBaseModel2.call(this, {
            field_type: _FormEditModel.field_types.AUTOCOMPLETE,
            loading: false, // 加载loading
            searchCodes: [], // 搜索结果
            onSearchChange: (0, _mobx.action)(function () {
                var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(value) {
                    var resp;
                    return regeneratorRuntime.wrap(function _callee$(_context) {
                        while (1) {
                            switch (_context.prev = _context.next) {
                                case 0:
                                    _context.prev = 0;

                                    _this3.searchCodes = [];
                                    _this3.loading = true;
                                    _context.next = 5;
                                    return _app2.default.http.searchCode({ q: value });

                                case 5:
                                    resp = _context.sent;

                                    _this3.loading = false;
                                    _this3.searchCodes = resp.data;
                                    _context.next = 13;
                                    break;

                                case 10:
                                    _context.prev = 10;
                                    _context.t0 = _context['catch'](0);

                                    console.log(_context.t0);

                                case 13:
                                    ;

                                case 14:
                                case 'end':
                                    return _context.stop();
                            }
                        }
                    }, _callee, _this4, [[0, 10]]);
                }));

                return function (_x4) {
                    return _ref.apply(this, arguments);
                };
            }())
        }));
    }

    return CRFAutoCompleteModel;
}(CRFFieldBaseModel);

;

var CRFRadioModel = exports.CRFRadioModel = function (_CRFFieldBaseModel3) {
    _inherits(CRFRadioModel, _CRFFieldBaseModel3);

    function CRFRadioModel(options) {
        var _getSettings;

        var _this5;

        _classCallCheck(this, CRFRadioModel);

        return _this5 = _possibleConstructorReturn(this, _CRFFieldBaseModel3.call(this, {
            field_type: _FormEditModel.field_types.RADIO,
            setting_title: [edit_title.BASE, edit_title.OPTIONS, edit_title.VAILDATE, edit_title.DEPEND],
            settings: getSettings((_getSettings = {}, _getSettings[edit_title.OPTIONS.type] = [settings.OPTIONSINLINE, settings.SETOPTIONS], _getSettings)),
            field_options: getFieldOptions({
                inline: true,
                options: [{
                    code: '0',
                    label: '0= 未知',
                    id: 'default_0'
                }, {
                    code: '1',
                    label: '1= 未知',
                    id: "default_1"
                }]
            }),
            changeOptionsIndex: function changeOptionsIndex(layouts) {
                var _options = (0, _utils.copy)(_this5.field_options.options);
                var _new_options = new Array(_options.length);

                var _loop = function _loop(i) {
                    var item = _options.filter(function (item) {
                        return item.id === layouts[i].i;
                    });
                    _new_options[layouts[i].y] = item[0];
                };

                for (var i = 0; i < layouts.length; i++) {
                    _loop(i);
                };
                return _new_options;
            },
            addOption: function addOption(id) {
                var _options = (0, _utils.copy)(_this5.field_options.options);
                var _index = _options.findIndex(function (item) {
                    return item.id === id;
                });
                var _code = _this5.getOptionCode(_options.length - 1, _options);
                _options.splice(_index + 1, 0, {
                    code: _code,
                    label: _code + '= 未知',
                    id: new Date().valueOf().toString()
                });
                return _options;
            },
            deleteOption: function deleteOption(id) {
                var _options = (0, _utils.copy)(_this5.field_options.options);
                var _index = _options.findIndex(function (item) {
                    return item.id === id;
                });
                _options.splice(_index, 1);
                return _options;
            },
            getOptionCode: function getOptionCode(code, options) {
                var _code = code + 1;
                var _index = options.findIndex(function (item) {
                    return item.code === _code + '';
                });
                if (_index !== -1) {
                    return this.getOptionCode(_code, options);
                } else {
                    return _code + '';
                }
            }
        }));
    }

    return CRFRadioModel;
}(CRFFieldBaseModel);

;

var MapType = function () {
    function MapType() {
        _classCallCheck(this, MapType);
    }

    MapType.prototype.getStore = function getStore(type) {
        switch (type) {
            case _FormEditModel.field_types.TEXT:
                return new CRFTextModel();
            case _FormEditModel.field_types.AUTOCOMPLETE:
                return new CRFAutoCompleteModel();
            case _FormEditModel.field_types.RADIO:
                return new CRFRadioModel();
            default:
                return new CRFTextModel();
        };
    };

    MapType.prototype.getView = function getView(type) {
        switch (type) {
            case _FormEditModel.field_types.TEXT:
                return _FieldView.TextView;
            case _FormEditModel.field_types.AUTOCOMPLETE:
                return _FieldView.AutoCompleteView;
            case _FormEditModel.field_types.RADIO:
                return _FieldView.RadioView;
            default:
                return _FieldView.TextView;
        };
    };

    return MapType;
}();

;

var mapType = exports.mapType = new MapType();