'use strict';

exports.__esModule = true;
exports.field_types = undefined;

var _mobx = require('mobx');

var _EditViews = require('./EditViews');

var _utils = require('../utils');

var _FieldDesignModel = require('./FieldDesignModel');

var _FieldDesignModel2 = _interopRequireDefault(_FieldDesignModel);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// {
//     name: '添加问题',
//     type: 'addField',
//     icon: 'tool',
//     view: AddFieldView
// },
var field_types = exports.field_types = {
    TEXT: 'text',
    AUTOCOMPLETE: 'autocomplete',
    CHECKBOX: 'checkbox',
    RADIO: 'radio',
    DATE: 'date',
    DROPDOWN: 'dropdown',
    LABEL: 'label'
};

var FormEditModel = function () {
    function FormEditModel() {
        var _this = this;

        _classCallCheck(this, FormEditModel);

        (0, _mobx.extendObservable)(this, {
            activeTab: 'layouts',
            isDrag: false,
            dragTop: 0,
            dragLeft: 0,
            dragWidth: 0,
            dragHeight: 0,
            dragIndex: 0,
            dragFieldType: '',
            _move: {
                x: 0,
                y: 0
            },
            ActiveView: _EditViews.LayoutsView,
            tabs: [{
                name: '排版布局',
                type: 'layouts',
                icon: 'solution',
                view: _EditViews.LayoutsView
            }, {
                name: '基本信息',
                type: 'base',
                icon: 'edit',
                view: _EditViews.BaseView
            }, {
                name: '数据集',
                type: 'mateFields',
                icon: 'book',
                view: _EditViews.MateFieldsView
            }, {
                name: '数据元',
                type: 'mate',
                icon: 'appstore-o',
                view: _EditViews.MateView
            }],
            fieldTypes: [{
                type: field_types.TEXT,
                title: '单行文本',
                note: '单行文本用于单行文字量较少的文本填空',
                icon: 'icon-typography'
            }, {
                type: field_types.AUTOCOMPLETE,
                title: "自动匹配文本",
                note: '自动匹配文本中的关键字',
                icon: 'icon-pen2'
            }, {
                type: field_types.RADIO,
                title: '单选题',
                note: '用于选取唯一答案(建议选项较少时使用)',
                icon: 'icon-circles'
            }, {
                type: field_types.CHECKBOX,
                title: '多选题',
                note: '用于选择多个答案',
                icon: 'icon-checkbox-partial2'
            }, {
                type: field_types.DATE,
                title: '日期选择',
                note: '用于时间日期的选取',
                icon: 'icon-calendar2'
            }],
            setFields: (0, _mobx.action)(function (fieldTypes) {
                var _fieldTypes = (0, _utils.copy)(_this.fieldTypes);
                if (!(0, _utils.isEmpty)(fieldTypes)) {
                    _fieldTypes = fieldTypes;
                };
                for (var i = 0; i < _fieldTypes.length; i++) {
                    _fieldTypes[i].draging = false;
                };
                _this.fieldTypes = _fieldTypes;
            }),
            changeTab: (0, _mobx.action)(function (type, view) {
                _this.activeTab = type;
                _this.ActiveView = view;
            }),
            startDrag: (0, _mobx.action)(function (index, type, e, node) {
                var _pos = {
                    x: 0,
                    y: 0
                };
                _this.getOffeset(_pos, node);
                _this.dragWidth = node.offsetWidth;
                _this.dragHeight = node.offsetHeight;
                _this._move.x = e.clientX;
                _this._move.y = e.clientY;
                _this.dragLeft = _pos.x;
                _this.dragTop = _pos.y;
                _this.dragFieldType = type;
                _this.changeDraging(index, true);
                _this.dragIndex = index;
            }),
            changeDraging: (0, _mobx.action)(function (index, type) {
                _this.isDrag = type;
                var _filedTypes = (0, _utils.copy)(_this.fieldTypes);
                _filedTypes[index].draging = type;
                _this.fieldTypes = _filedTypes;
            }),
            stopDrag: (0, _mobx.action)(function (e) {
                if (_this.isDrag) {
                    _FieldDesignModel2.default.addField(_this.dragFieldType);
                    _this.changeDraging(_this.dragIndex, false);
                    _FieldDesignModel2.default.setElementInitCancel();
                };
            }),
            onMove: (0, _mobx.action)(function (e) {
                if (_this.isDrag) {
                    var _move = {
                        x: e.clientX - _this._move.x,
                        y: e.clientY - _this._move.y
                    };
                    _this._move = {
                        x: e.clientX,
                        y: e.clientY
                    };
                    _this.dragLeft = _this.dragLeft + _move.x;
                    _this.dragTop = _this.dragTop + _move.y;
                    _FieldDesignModel2.default.changeElementInit(_this.dragLeft, _this.dragTop);
                };
            })
        });
    }

    FormEditModel.prototype.getOffeset = function getOffeset(_pos, node) {
        _pos.x = _pos.x + node.offsetLeft;
        _pos.y = _pos.y + node.offsetTop;
        if (node.offsetParent) {
            this.getOffeset(_pos, node.offsetParent);
        };
    };

    return FormEditModel;
}();

;

var crfFormEdit = new FormEditModel();

exports.default = crfFormEdit;