"use strict";

exports.__esModule = true;
exports.isEmpty = isEmpty;
exports.copy = copy;
exports.keys = keys;
exports.values = values;
exports.treeToObj = treeToObj;
exports.getValueByDeepName = getValueByDeepName;
exports.setByName = setByName;
var isArray = exports.isArray = function isArray(arr) {
    return is(arr, "Array");
};
var isObject = exports.isObject = function isObject(obj) {
    return is(obj, "Object");
};
var isString = exports.isString = function isString(str) {
    return is(str, "String");
};
var isNumber = exports.isNumber = function isNumber(num) {
    return is(num, "Number");
};
var isFnc = exports.isFnc = function isFnc(fnc) {
    return is(fnc, "Function");
};
var isUndefined = exports.isUndefined = function isUndefined(und) {
    return is(und, 'Undefined');
};
var isNull = exports.isNull = function isNull(_null) {
    return is(_null, 'Null');
};

/**
 * @name 判断变量是否为空
 * @param {*} t
 * 如果是undefined或null 返回true
 * 如果是string 判断是否为空字符串
 * 如果是number 判断是否为NaN
 * 如果是Object或Array 判断是否有成员
 * 如果非上述类型 返回false 
 */
function isEmpty(t) {
    if (isUndefined(t)) {
        return true;
    };
    if (isNull(t)) {
        return true;
    }
    if (isString(t)) {
        return t === '';
    };
    if (isNumber(t)) {
        return t === NaN;
    };
    if (isObject(t)) {
        if (isArray(t)) {
            return t.length === 0;
        };
        for (var key in t) {
            return false;
        };
        return true;
    };
    return false;
};

function copy(obj) {
    if (isObject(obj)) {
        if (!isUndefined(obj.length)) {
            var _arr = [];
            for (var i = 0; i < obj.length; i++) {
                _arr.push(obj[i]);
            };
            return _arr;
        };
        var _obj = {};
        for (var key in obj) {
            _obj[key] = obj[key];
        };
        return _obj;
    };
    return obj;
};

function keys(obj) {
    var _keys = [];
    if (!isObject(obj)) {
        return _keys;
    };
    for (var key in obj) {
        _keys.push(key);
    };
    return _keys;
};

function values(obj) {
    var _values = [];
    if (!isObject(obj)) {
        return _values;
    };
    for (var key in obj) {
        _values.push(obj[key]);
    };
    return _values;
};

function treeToObj(arr) {
    var _transform = function _transform(obj, arr) {
        for (var i = 0; i < arr.length; i++) {

            obj[arr[i].key] = {};
            for (var key in arr[i]) {
                if (key !== 'children') {
                    obj[arr[i].key][key] = arr[i][key];
                };
            };
            if (arr[i].children && arr[i].children.length) {
                _transform(obj, arr[i].children);
            }
        };
    };
    var _obj = {};
    _transform(_obj, arr);
    return _obj;
};

function getValueByDeepName(obj, name) {
    if (isEmpty(name)) {
        return obj;
    };
    var _nameArr = name.split('.');
    if (_nameArr.length > 1) {
        var _v = void 0;
        for (var i = 0; i < _nameArr.length; i++) {
            _v = i === 0 ? obj[_nameArr[i]] : _v[_nameArr[i]];
        };
        return _v;
    };
    return obj[name];
};

function setByName(obj, name, value) {
    var _obj = copy(obj);
    var _nameArr = name.split('.');
    if (_nameArr.length > 1) {
        var _f = {};
        for (var i = _nameArr.length - 1; i >= 0; i--) {
            var _ref, _ref2;

            var _arr = [];
            for (var j = 0; j < i; j++) {
                _arr.push(_nameArr[j]);
            };
            var _p = getValueByDeepName(_obj, _arr.join('.'));
            if (_p.length) {
                _p[_nameArr[i]] = _f;
                _f = _p;
                continue;
            };
            _f = i === _nameArr.length - 1 ? (_ref = {}, _ref[_nameArr[i]] = value, _ref) : (_ref2 = {}, _ref2[_nameArr[i]] = _f, _ref2);
            _f = Object.assign({}, getValueByDeepName(_obj, _arr.join('.')), _f);
        };
        _obj = Object.assign({}, _obj, _f);
    } else {
        var _Object$assign;

        _obj = Object.assign({}, _obj, (_Object$assign = {}, _Object$assign[name] = value, _Object$assign));
    };
    return _obj;
};

function is(obj, type) {
    return type === "Null" && obj === null || type === "Undefined" && obj === void 0 || type === "Number" && isFinite(obj) || Object.prototype.toString.call(obj).slice(8, -1) === type;
};