import axios from 'axios';
import qs from 'qs';
import { Modal } from 'antd';

export default class Api {
    _base;
    post( url, data ){
        return this._base.post(url, qs.stringify(data) ).catch(error => {
            let message = '系统错误';
            if (error.response) {
                message = error.response.data.message;
            } else if (error.request) {
                message = error.request;
            } else {
                message = error.message
            };
            Modal.error({
                title: '系统提示',
                content: message
            });
        });
    };
    get( url, data ){
        return this._base.get(url, { params: data }).catch(error => {
            let message = '系统错误';
            if (error.response) {
                message = error.response.data.message;
            } else if (error.request) {
                message = error.request;
            } else {
                message = error.message
            };
            Modal.error({
                title: '系统提示',
                content: message
            });
        });
    };

    constructor({ baseUrl='', apis={}}){
        this._axios = axios;

        this._base = axios.create({
            baseURL: baseUrl
        });

        for( let key in apis ){
            if( apis[key].mode === 'get' ){
                this[key] = ( data = {} ) => {
                    return this.get( apis[key].url, data )
                };
            }else{
                this[key] = ( data = {} ) => {
                    return this.post( apis[key].url, data )
                };
            };    
        };
    };
};