import * as React from 'react';
import { Tree, Icon, Button, Popover, Radio } from 'antd';
import { design_types , title_position} from './FieldDesignModel';

const { TreeNode } = Tree;
const RadioGroup = Radio.Group;

export class InputWarp extends React.Component {
    render(){
        const { label } = this.props;
        return (
            <div 
                style={{ 
                    display: 'table',
                    width: '100%',
                    padding: '8px 0' 
                }}
            >
                <div
                    style={{
                        display: 'table-cell',
                        textAlign: 'right'
                    }}
                >
                    <span
                        style={{
                            fontSize: '12px',
                            color: '#333'
                        }}
                    >{ label }</span>
                </div>
                <div
                    style={{
                        display: 'table-cell',
                        width: '200px',
                        paddingLeft: '10px'
                    }}
                >
                {
                    this.props.children
                }
                </div>
            </div>
        );
    };
};

export class FormViewDesign extends React.Component {
    render(){
        const { 
            data,
            getName,
            addNode,
            deleteNode 
        } = this.props;
        const loop = data => {
            return data.map((item, i) => {
                let title;
                switch( item.type ){
                    case design_types.ROW:
                        title = '行';
                        break;
                    case design_types.COLUMN:
                        title = '列';
                        break;
                    case design_types.PAGE:
                        title = '表';
                        break;
                    case design_types.SPACE:
                        title = '空';
                        break;
                    default :
                        title = '行';
                        break;
                };
                
                const _view = key => (
                    <p 
                        style={{
                            paddingLeft: "4px"
                        }} 
                    >
                        <span>{title}{
                            item.children.length
                            ?   `( ${ item.children.length } )`
                            :   void 0
                        }</span>
                        {
                            item.type === design_types.COLUMN
                            ?   void 0
                            :   <AddRowComponent onOk={ addNode } id={ key } type={ item.type } />   
                        }
                        <span
                            style={{
                                marginLeft: item.type === design_types.COLUMN ? '24px' : '8px'
                            }}
                        >
                            <Button 
                                type="danger" 
                                size='small' 
                                shape="circle" 
                                icon='delete'
                                disabled={  item.disabled }
                                onClick={() => deleteNode(key)}
                            />
                        </span>
                    </p>
                );
                if(item.children && item.children.length){
                    return (
                        <TreeNode key={item.key}  title={ _view(item.key) } >
                        {
                            loop(item.children)
                        }
                        </TreeNode>
                    );
                };
                return (
                    <TreeNode key={item.key} title={ _view(item.key) } />
                );
            });
        };
        return (
            <div>
                <Tree
                    style={{ width: '100%' }}
                    //draggable
                    showLine
                    defaultExpandAll
                >
                {
                    loop(data)
                }
                </Tree>
            </div>
        );
    };
};

class AddRowComponent extends React.Component {
    constructor(props){
        super(props);
        this.state = {
            type: props.type === design_types.ROW ? design_types.COLUMN : design_types.ROW,
            visible: false,
            position: title_position.TOP
        };
    };
    render(){
        const { type } = this.props;
        return(
            <span
                style={{
                    marginLeft: '24px'
                }}
                //onClick={() => addNode( key )}
            >
                <Button 
                    size='small' 
                    shape="circle" 
                    icon='file-add'
                    onClick={() => {
                        this.setState({ visible: !this.state.visible })
                    }}
                />
                <Popover
                    placement='bottom'
                    title='添加'
                    trigger='click'
                    visible={ this.state.visible }
                    content={(
                        <div 
                            style={{margin: '0 -16px'}}
                        >
                            <div
                                style={{ padding: '0 16px' }}
                            >
                                <RadioGroup
                                    value={this.state.type}
                                    onChange={(e) => this.setState({ type: e.target.value })}
                                >
                                    <Radio value={ design_types.COLUMN } disabled={ type !== design_types.ROW }  >列</Radio>
                                    <Radio value={ design_types.ROW }>行</Radio>
                                    <Radio value={ design_types.PAGE } >表</Radio>
                                    <Radio value={ design_types.SPACE } >空</Radio>
                                </RadioGroup>
                                {
                                    this.state.type === design_types.PAGE
                                    ?   <div
                                            style={{ marginTop: '8px',paddingTop: '8px',borderTop: '1px solid #ddd' }}
                                        >
                                            <span style={{ marginRight: '8px' }} >标题位置</span>
                                            <RadioGroup
                                                value={ this.state.position }
                                                onChange={(e) => this.setState({ position: e.target.value })}
                                            >
                                                <Radio value={ title_position.TOP }>上</Radio>
                                                <Radio value={ title_position.LEFT } >左</Radio>
                                            </RadioGroup>
                                        </div> 
                                    :   void 0  
                                }
                            </div>
                            <div
                                style={{
                                    marginTop: '8px',
                                    padding: '0 16px',
                                    paddingTop: '8px',
                                    overflow: 'hidden',
                                    borderTop: '1px solid #ddd'
                                }}
                            >
                                <div 
                                    style={{ float: 'right' }}
                                >
                                    <Button 
                                        size='small' 
                                        style={{ marginRight: '8px' }} 
                                        onClick={() => this.setState({ visible: false, type: design_types.COLUMN, position: title_position.TOP })}
                                    >取消</Button>
                                    <Button 
                                        size='small' 
                                        type='primary' 
                                        onClick={() => {
                                            this.props.onOk(this.props.id, this.state.type, this.state.position)
                                            this.setState({ visible: false, type: design_types.COLUMN, position: title_position.TOP })
                                        }}
                                    >确定</Button>
                                </div>
                            </div>
                        </div>
                    )}
                >
                </Popover>
            </span>  
        );
    };
};