import * as React from 'react';
import { inject, observer } from 'mobx-react';
import {design_types, title_position} from './FieldDesignModel';
import FieldView from './FieldView';
import {isUndefined} from '../utils';
import {Input} from "antd";

const FieldContent = inject('crfDesign', 'crfFormEdit')(observer(class extends React.Component {
    componentWillMount(){
        const { name, headerStyle } = this.props;
        this.props.crfDesign.init({
            name,
            headerStyle
        })
    }
    render(){
        const {
            formName,
            headerStyle,
            design,
            setCellPosition,
            getField,
        } = this.props.crfDesign;
        const {
            isDrag
        } = this.props.crfFormEdit;
        const { width } = this.props;
        const _view = col => {
            return col.map((item, i) => {
                const _className = 'crf-design-' + item.type;
                if( item.children && item.children.length ){
                    if( item.type === design_types.PAGE ){
                        return (
                            <div className={ _className + ' ' + item.titlePosition } >
                                <div></div>
                                <div>
                                    <div className='crf-design-table' >
                                    {
                                        _view( item.children )
                                    }
                                    </div>
                                </div>
                            </div>
                        );
                    };
                    return (
                        <div className={ _className }  key={ item.key }>
                            { _view( item.children ) }
                        </div>
                    );
                };
                if( item.type === design_types.SPACE ){
                    return (
                        <div className={ _className } style={ item.spaceNoBorder ? { border: '0 none' } : void 0 } >
                        </div>
                    );
                };
                return (
                    <Cell
                        width={ `${(width - 32)/ col.length }px` }
                        className={ _className }
                        key={ item.key }
                        item = {item}
                        index={ i }
                        last={ col.length - 1 }
                        isDrag = { isDrag }
                        getField = { getField }
                        setCellPosition={ setCellPosition }
                    />
                );
            });
        };
        return (
            <div className='crf-field-container'>
                <div className='field-header'>
                    <h4
                        style={ headerStyle }
                    >{ formName }</h4>
                </div>
                <div className='crf-design-table'>
                    <div style={{ height:0, width: '100%', borderBottom: '2px dashed #ddd' }} ></div>
                {
                    _view( design )
                }
                </div>
            </div>
        );
    };
}));


class Cell extends React.Component {
    componentDidMount(){
        this.props.setCellPosition(this.props.item.key, this._cell)
    }
    componentWillUpdate(nextProps){
        if( this.props.isDrag !== nextProps.isDrag && nextProps.isDrag ){
            this.props.setCellPosition(this.props.item.key, this._cell)
        }
    }
    render(){
        const {
            elementInit,
            field_id
        } = this.props.item;
        const {
            className,
            getField,
            width,
            index,
            last
        } = this.props;
        let store, view
        if( !isUndefined(field_id) ){
            const _model = getField(field_id);
            store = _model.store;
            view = _model.view;
        }
        return (
            <div
                className={ className } 
                style={ Object.assign({},{ width, verticalAlign: 'top' }, elementInit ? { backgroundColor: 'rgba(250, 250, 250, 1)' } : void 0)}
            >
                <div ref={ _cell => this._cell = _cell } style={{ position: 'relative', minHeight: isUndefined(field_id) ? '80px' : 0 }} >
                {
                    index !== last
                    ?   <div className='right-line'>
                            <input />
                        </div>
                    :   void 0
                }
                {
                    isUndefined(field_id)
                    ?   void 0
                    :   <FieldView
                            store={ store }
                            SubView={ view }
                            disabled={true}
                            mode='DEV'
                        />
                }
                </div>
            </div>
        );
    };
};

export default FieldContent
