import * as React from 'react';
import { inject, observer } from 'mobx-react';
import ReactGridLayout from 'react-grid-layout';
import {isUndefined, isEmpty} from '../utils';
import FieldView from './FieldView';
import 'react-grid-layout/css/styles.css';
import { Icon, Popover, Radio } from 'antd';
import FieldSettingView from './FieldSetting';
//const ResponsiveReactGridLayout = WidthProvider(Responsive);

const RadioGroup = Radio.Group;

const FieldContent = inject('crfDesign')(observer(class extends React.Component{
    constructor(props){
        super(props);
        this.state = {
            width: 0
        };
    };
    componentWillMount(){
        const { name, headerStyle } = this.props;
        this.props.crfDesign.init({
            name,
            headerStyle
        });
    };
    componentDidMount(){
        this.setState({width: this._content.clientWidth})
    }
    render(){
        const {
            formName,
            headerStyle,
            layouts,
            getField,
            layoutsChange,
            setCellNode,
            setCellPosition,
            deleteCell,
            fieldsCollection,
            changeFieldCollection,
            changeCellContentVertical,
            border,
            addRelation,
            is_static,
        } = this.props.crfDesign;
        const { mode } = this.props;
        return (
             <div className='crf-field-container'>
                <div className='field-header'>
                    <h4
                        style={ headerStyle }
                    >{ formName }</h4>
                </div>
                <div
                    ref={content => this._content = content}
                >
                    <ReactGridLayout
                        className="layout"
                        style={{ backgroundColor: border.show ? border.color : 'transparent' }} 
                        cols={12}
                        rowHeight={12}
                        margin={[2, 2]}
                        width={ this.state.width }//this._content ? this._content.clientWidth : void 0}
                        onLayoutChange = { layout => layoutsChange(layout) }
                        //onDragStop={ layout => layoutsChange(layout) }
                        //onResizeStop={ layout => layoutsChange(layout) }
                    >
                    {
                        layouts.map(item => {
                            if(mode !== 'DEV' || is_static){
                                item.layout.static = true;
                            };
                            return(
                                <div 
                                    key={ item.key } 
                                    data-grid={ item.layout }
                                    style={ item.style }
                                >
                                    <Cell
                                        verticalAlign = { item.cellVertical }
                                        index = { item.key }
                                        setCellNode = { setCellNode }
                                        width = { this.state.width }//this._content ? this._content.clientWidth : void 0 }
                                        setCellPosition={ setCellPosition }
                                        le = { layouts.length }
                                        getField = { getField }
                                        field_id = { item.field_id }
                                        mode = { mode }
                                        deleteCell = { deleteCell }
                                        fieldsCollection={ fieldsCollection }
                                        changeFieldCollection={ changeFieldCollection }
                                        changeCellContentVertical={ changeCellContentVertical }
                                        addRelation={ addRelation }
                                    />
                                </div>
                            )
                        })
                    }
                    </ReactGridLayout>
                </div>
            </div>
        );
    };
}));

class Cell extends React.Component {
    componentDidMount(){
        if( this.props.mode === 'DEV' ){
            this.props.setCellNode(this.props.index, this._cell)
        }
    }
    componentWillUpdate(nextProps){
        if( this.props.mode === 'DEV' && (this.props.width !== nextProps.width || this.props.le !== nextProps.le)){
            this.props.setCellPosition( this.props.index )
        }
    }
    render(){
        const {
            getField,
            field_id,
            mode,
            index,
            deleteCell,
            fieldsCollection,
            changeFieldCollection,
            verticalAlign,
            changeCellContentVertical,
            addRelation
        } = this.props;
        let store, view;
        if( !isEmpty(field_id) ){
            const _model = getField(field_id);
            store = _model.store;
            view = _model.view;
        }
        return (
            <div
                style={{ width: "100%", height: '100%', overflow: 'auto', position: 'relative' }}
                ref={ cell => this._cell = cell }
                onMouseDown = {() => addRelation(field_id)}
            >
            {
                mode === 'DEV'
                ?   <div
                        className='crf-cell-edit-btns'
                        style={{ width: isEmpty(field_id) ? '50px' : '75px' }}
                    >
                    {
                        isEmpty(field_id)
                        ?   void 0
                        :   <FieldSettingView store={ store } onChange={ value => changeFieldCollection(field_id, value) } />   
                    }
                        <Popover
                            placement='top'
                            trigger='click'
                            content={(
                                <div>
                                    <RadioGroup 
                                        value={ verticalAlign }
                                        onChange={ e => changeCellContentVertical(index, e.target.value) } 
                                    >
                                        <Radio value='top'>居上</Radio>
                                        <Radio value='middle'>居中</Radio>
                                        <Radio value='bottom'>居下</Radio>
                                    </RadioGroup>
                                </div>
                            )}
                        >
                            <span>
                                <Icon type="layout" />
                            </span>
                        </Popover>
                        <span
                            onClick={() => deleteCell(index)}
                        >
                            <Icon type="delete" />
                        </span>
                    </div> 
                : void 0      
            }
            {
                isEmpty(field_id)
                ?   void 0
                :   <div style={{ display: 'table', width: '100%', height: '100%' }} >
                        <div style={{ display: 'table-cell', verticalAlign }} >
                            <FieldView
                                store={ store }
                                SubView={ view }
                                disabled={mode === 'DEV'}
                                mode={ mode }
                                onChange={ value => changeFieldCollection(field_id, value) }
                            />
                        </div>
                    </div>
            }
            </div>
        );
    };
};


export default FieldContent;