import { extendObservable, action } from 'mobx';
import { BaseFormModel } from './FormModel';
import { copy, isUndefined, treeToObj} from '../utils';
import { 
    CRFTextModel , 
    mapType
} from './FieldModel';
import crfFormEdit from './FormEditModel';

export const design_types = {
    ROW: 'row',
    PAGE: 'page',
    SPACE: 'space',
    COLUMN: 'column'
};

export const title_position = {
    TOP: 'top',
    LEFT: 'left',
    RIGHT: 'right'
}

class FieldDesignModel extends BaseFormModel {
    constructor(){
        super();
        extendObservable(this, {
            formName: '',
            selectKey: '',
            mainElement: null,
            design: [{
                type: design_types.ROW,
                key: '0',
                disabled: true,
                children: [{
                    type: design_types.COLUMN,
                    key: '0_0',
                    style: {
                        width: '100%',
                        borderLeft: '2px solid #ddd',
                        borderBottom: '2px solid #ddd',
                        borderRight: '2px solid #ddd'
                    },
                    disabled: true,
                    children: [ ]
                }]
            }],
            fieldsCollection: [ ],
            headerStyle: {
                fontSize: '18px',
                textAlign: 'center',
                fontWeight: 400,
                color: '#333'
            },
            fields: [ ],
            init: action(({ name, headerStyle }) => {
                //this.design = [ ];
                this.formName = name;
                if( headerStyle ){
                    this.headerStyle = Object.assign({}, this.headerStyle, headerStyle)
                }
            }),
            changeFormName: action(name => {
                this.change = true;
                this.formName = name;
            }),
            changeHeaderStyle: action(style => {
                this.change = true;
                this.headerStyle = Object.assign({}, this.headerStyle, style);
            }),
            setDesign: action(design => this.design = design),
            addRow: action((key, type, position) => {
                this.change = true;
                const _design = copy(this.design);
                const _this = this.getParent(key, _design);
                let _parent, _key = key;
                if( type === design_types.COLUMN ){
                     _parent = this.getParent(key, _design).children;
                }else{
                    let _keyArr = key.split('_');
                    _keyArr.splice(-1, 1);
                    _parent = _design;
                    _key = _keyArr.join('_');
                    if( _keyArr.length ){
                        _parent = this.getParent(_key, _design).children;
                    }
                }
                const _index = _parent.findIndex(item => item.key === _this.key);
                const _width = crfFormMain.page.width;
                console.log( _width ) 
                let _item = {
                    type: type,
                    key: '',
                    children: [ ]
                };
                if( type == design_types.ROW ){
                    _item.children.push({
                        type: design_types.COLUMN,
                        key: '',
                        children: [ ]
                    })
                };
                if( type === design_types.PAGE ){
                    _item.titlePosition = position;
                    _item.children.push({
                        type: design_types.ROW,
                        key: '',
                        children: [{
                            type: design_types.COLUMN,
                            key: '',
                            children: []
                        }]
                    });
                };
                if( type === design_types.COLUMN ){
                    _parent.push( _item );
                }else{
                    _parent.splice(_index + 1, 0, _item);
                };
                this.setKey(_design);
                this.setDesign(_design);
            }),
            deleteRow: action(key => {
                this.change = true;
                const _design = copy(this.design);
                let _this = this.getParent(key, _design);
                let _keyArr = key.split('_');
                _keyArr.splice(-1, 1);
                let _parent = _design
                if( _keyArr.length ){
                    _parent = this.getParent(_keyArr.join('_'), _design).children;
                }
                const _index = _parent.indexOf( _this );
                _parent.splice(_index, 1);
                this.setKey(_design);
                this.setDesign(_design);
            }),
            setCellPosition: action(( key, node ) => {
                let _pos = {
                    x: 0,
                    y: 0
                };
                crfFormEdit.getOffeset(_pos, node);
                const _width = node.offsetWidth;
                const _Height = node.offsetHeight;
                const _area = {
                    x:{
                        min: _pos.x,
                        max: _pos.x + _width
                    },
                    y:{
                        min: _pos.y,
                        max: _pos.y + _Height
                    }
                };
                let _design = copy(this.design);
                let _parent = this.getParent(key, _design);
                _parent.area = _area;
                this.setDesign(_design)
            }),
            changeElementInit: action((x, y, w, h) => {
                this.selectKey = ''
                let _design = copy(this.design);
                const _tree = treeToObj(_design);
                const _top = this.mainElement.scrollTop;
                const _left = this.mainElement.scrollLeft;
                for(let key in _tree){
                    if(_tree[key].type !== design_types.COLUMN){
                        continue;
                    };
                    let _x = x >= (_tree[key].area.x.min - _left) && x <= (_tree[key].area.x.max - _left);
                    let _y = y >= (_tree[key].area.y.min - _top) && y <= (_tree[key].area.y.max - _top);
                    if( _x && _y ){
                        this.selectKey = key;
                    };
                    let _parent = crfDesign.getParent(key, _design);
                    _parent.elementInit = ( _x && _y );
                };
                this.setDesign(_design);
            }),
            setElementInitCancel: action(() => {
                let _design = copy(this.design);
                const _tree = treeToObj(_design);
                for(let key in _tree){
                    if(_tree[key].type !== design_types.COLUMN){
                        continue;
                    };
                    let _parent = crfDesign.getParent(key, _design);
                    _parent.elementInit = false;
                };
                this.setDesign(_design);
            }),
            addField: action(( type ) => {
                if( this.selectKey === '' ){
                    return void 0;
                };
                const _store = mapType.getStore(type);
                _store.setFieldOption({id: this.fieldsCollection.length});
                let _fields = copy(this.fieldsCollection);
                _fields.push( _store );
                this.fieldsCollection = _fields;
                let _design = copy(this.design);
                let _parent = this.getParent(this.selectKey, _design);
                _parent.field_id = _store.id;
                this.setDesign(_design);
            }),
            getField: id => {
                const _store = this.fieldsCollection.filter(item => item.id === id);
                return {
                    store: _store[0],  
                    view: mapType.getView( _store[0].type )
                };
            },
            setMainElement: action(_el => {
                this.mainElement = _el;
            })
        });
    };

    getParent(key, _design){
        let _keys = key.split('_');
        let _parent;
        for(let i = 0; i < _keys.length; i ++){
            if( i === 0 ){
                _parent = _design[_keys[i]];
            }else{
                _parent = _parent.children[_keys[i]];
            };
        };
        return _parent;
    };

    setKey(arr, k){
        for(let i = 0; i < arr.length; i++){
            let _key = i + '';
            if( !isUndefined(k) ){
                _key = k + '_' + i;
            };
            arr[i].spaceNoBorder = false;
            if( arr[i].type === design_types.SPACE && (i === arr.length - 1 || arr[i + 1].type === design_types.SPACE) ){
                arr[i].spaceNoBorder = true;
            };
            arr[i].disabled = false;
            if( arr.length === 1 ){
                arr[i].disabled = true;
            };
            arr[i].key = _key;
            if( arr[i].children && arr[i].children.length ){
                this.setKey(arr[i].children, _key);
            };
        };
    };

    get fontSizeOptions(){
        let _arr = [ ];
        for( let i = 12; i < 64; i++ ){
            _arr.push( i );
        };
        return _arr
    };

    get fontWeightOptions(){
        return [
            'normal',
            '400',
            '500',
            '600',
            'bold'
        ];
    };

    fieldName( id ){
        const _field = this.fields.filter(item => item.id === id);
        if( _field.length > 0 ){
            return _field[0].label;
        };
        return '未定义';
    };

    get textAlignOptions(){
        return [{
            value: 'left',
            label: '左'
        },{
            value: 'center',
            label: '中'
        },{
            value: 'right',
            label: '右'
        }];
    };

};

const crfDesign = new FieldDesignModel();

export default crfDesign;