import * as React from 'react';
import { observer } from 'mobx-react';
import ReactGridLayout from 'react-grid-layout';
import {
    Switch,
    Input,
    InputNumber,
    Popover,
    Icon,
    Radio,
    Button,
    message,
    Alert,
} from 'antd';

const RadioGroup = Radio.Group;

class FieldSettingView extends React.Component{
    render(){
        const { 
            settings,
            setting_title,
            setting_selected,
        } = this.props.store;
        const { onChange } = this.props;
        const _title=(
            <div style={{ display: 'table', width: '100%' }} >
                <div
                    style={{
                        display: 'table-cell', 
                        width: `${26 * setting_title.length + 4}px`, 
                        overflow: 'hidden',
                        verticalAlign: 'top',
                        paddingRight: '4px'
                    }}
                >
                {
                    setting_title.map(item => (
                        <div 
                            style={{ 
                                float: 'left', 
                                width: '26px', 
                                textAlign:"center",
                                padding: '4px 0',
                            }} 
                            key={ item.type }
                        >
                            <a
                                style={{
                                    display: 'block',
                                    width: '26px',
                                    margin: '0 auto',
                                    color: '#666',
                                    borderRadius: '2px',
                                    lineHeight: '24px', 
                                    backgroundColor:  item.type === setting_selected ? 'rgba(150, 150, 150, .2)' : 'transparent'
                                }}
                                onClick={() => onChange({ setting_selected:item.type})}
                            >
                                <i className={ item.icon } style={{ fontSize: '14px'}} ></i>
                            </a>
                        </div>
                    ))
                }
                </div>
                <div
                    style={{ 
                        display: 'table-cell', 
                    }}
                >
                    <p>
                    {
                        setting_title.filter(item => item.type === setting_selected)[0].label
                    }
                    </p>
                </div>
            </div>
        );
        const _content = (
            <div style={{width: '240px', height: '180px', overflowY: 'scroll', marginRight: '-16px', paddingRight: '16px'}} >
            {
                settings[setting_selected].map((View, i) => (
                    <View key={ i } store={ this.props.store } onChange = { onChange } />
                ))
            }
            </div>
        );
        return (
            <Popover
                title={ _title }
                content={ _content }
                placement='left'
                arrowPointAtCenter={ true }
                trigger='click'
            >
                <span>
                    <Icon type='edit' />
                </span>
            </Popover>
        );
    };
};

class InputType extends React.Component {
    render(){
        const {
            label,
            store,
            name
        } = this.props;
        return (
            <div className='crf-setting-wrap' >
                <label className='setting-title' >{ label }: </label>
                <div
                    className='setting-input'
                >
                    <Input
                        value={ store.getOptions(name) }
                        onChange={(e) => {
                            this.props.onChange({
                                [name]: e.target.value
                            })
                        }}
                    />
                </div>
            </div>
        );
    };
};

class NumberType extends React.Component{
    render(){
        const {
            label,
            store,
            name
        } = this.props;
        return (
            <div className='crf-setting-wrap' >
                <label className='setting-title' >{ label }: </label>
                <div
                    className='setting-input'
                >
                    <InputNumber
                        style={{ width: "100%" }}
                        value={ store.getOptions(name) }
                        onChange={value => {
                            this.props.onChange({
                                [name]: value
                            })
                        }}
                    />
                </div>
            </div>
        );
    };
};

class SwitchType extends React.Component {
    render(){
        const {
            label,
            store,
            name,
            checkedLabel,
            unCheckedLabel
        } = this.props;
        return(
            <div className='crf-setting-wrap' >
                <label className='setting-title' >{ label }: </label>
                <div
                    className='setting-input'
                >
                    <div style={{ float: 'left' }} >
                        <Switch
                            checked={ store.getOptions(name) }
                            onChange={checked => {
                                this.props.onChange({
                                    [name]: checked
                                })
                            }}
                            checkedChildren={ checkedLabel || '是' }
                            unCheckedChildren={ unCheckedLabel || '否' }
                        />
                    </div>
                </div>
            </div>
        );
    };
};

class RadioType extends React.Component{
    render(){
        const {
            label,
            store,
            name,
            options
        } = this.props;
        let _value;
        switch(store.getOptions(name)){
            case true:
                _value = 'true';
                break;
            case false:
                _value = 'false';
                break;
            default:
                _value = store.getOptions(name);
                break; 
        };
        return (
            <div className='crf-setting-wrap' >
                <label className='setting-title' >{ label }: </label>
                <div
                    className='setting-input'
                >
                    <RadioGroup 
                        value={ _value }
                        options={ options }
                        onChange={e => {
                            let _val;
                            switch(e.target.value){
                                case 'true':
                                    _val = true;
                                    break;
                                case 'false':
                                    _val = false;
                                    break;
                                default:
                                    _val = e.target.value;
                                    break;
                            };
                            this.props.onChange({[name]: _val});
                        }} 
                    />
                </div>
            </div>
        );
    };
};

class TextAreaType extends React.Component {
    render(){
        const {
            label,
            store,
            name
        } = this.props;
        return (
            <div className='crf-setting-wrap' >
                <label 
                    className='setting-title'
                    style={{ verticalAlign: "top" }}
                >{ label }: </label>
                <div
                    className='setting-input'
                >
                    <Input 
                        type='textarea'
                        value={ store.getOptions(name) }
                        onChange={e => {
                            this.props.onChange({
                                [name]: e.target.value
                            })
                        }}
                    />
                </div>
            </div>
        );
    };
};

class SetOptionsType extends React.Component{
    render(){
        const {
            label,
            store,
            name,
            onChange,
        } = this.props;
        return (
            <div>
                <div style={{ paddingLeft: '5px', paddingBottom: '4px' }}>{label}</div>
                <ReactGridLayout 
                    cols={1}
                    rowHeight={40}
                    margin={[2, 2]}
                    width={ 210 }
                    onDragStop={ layout => {
                        onChange({
                            [name]: store.changeOptionsIndex(layout)
                        });
                    }}
                >
                {
                    store.getOptions(name).map((item, i) => {
                        let _layouts = { x: 0, y: i, w: 1, h: 1, isResizable: false };
                        return(
                            <div
                                style={{backgroundColor: '#f0f0f0', paddingRight: '4px', paddingLeft: '2px', borderRadius: '2px'}} 
                                key={ item.id } 
                                data-grid={_layouts}

                            >
                                <div style={{ display: 'table', width: '100%', height: '100%' }}>
                                    <div style={{ display: 'table-cell', textAlign: 'center', cursor: 'pointer',width: '16px', verticalAlign: 'middle' }}>
                                        <i style={{color: '#aaa'}} className='icon-more'></i>
                                    </div>
                                    <div style={{ display: 'table-cell', width: '34px', verticalAlign: 'middle', paddingRight: '2px' }}>
                                        <Input
                                            style={{ textAlign: 'center' }} 
                                            value={item.code }
                                            onMouseDown={e => e.stopPropagation()}
                                            onChange={e => {
                                                onChange({
                                                    [`${name}.${i}.code`]: e.target.value
                                                })
                                            }}
                                        />
                                    </div>
                                    <div style={{ display: 'table-cell', verticalAlign: 'middle', height: '100%' }} >
                                        <Input 
                                            value={item.label}
                                            onMouseDown={e => e.stopPropagation()}
                                            onChange={e => {
                                                onChange({
                                                    [`${name}.${i}.label`]: e.target.value
                                                })
                                            }}
                                        />
                                    </div>
                                    <div style={{ display: 'table-cell', paddingLeft: '2px',verticalAlign: 'middle', width: '24px' }} >
                                        <Button 
                                            icon='plus'
                                            onMouseDown={e => e.stopPropagation()}
                                            onClick={() => 
                                                onChange({
                                                    [name]: store.addOption(item.id)
                                                })
                                            }
                                        />
                                    </div>
                                    <div style={{ display: 'table-cell', paddingLeft: '2px',verticalAlign: 'middle', width: '24px' }} >
                                        <Button 
                                            type='danger' 
                                            icon='delete'
                                            onMouseDown={e => e.stopPropagation()}
                                            onClick={() =>{
                                                if( store.getOptions(name).length < 2 ){
                                                    return message.error('选项不能少于一项！');
                                                }; 
                                                onChange({
                                                    [name]: store.deleteOption(item.id)
                                                });
                                            }}
                                        />
                                    </div>
                                </div>
                            </div>
                        );
                    })
                }
                </ReactGridLayout>
            </div>
        );
    };
};

class CodeType extends React.Component {
    render(){
        const {
            label,
            store,
            name,
            onChange,
        } = this.props;
        return(
            <div>
                <div className='crf-setting-wrap' >
                    <label className='setting-title' >{ label }:</label>
                    <div className='setting-input'>
                        <Button 
                            size='small' 
                            type='danger' 
                            icon='plus-square'
                            onClick={() => onChange({[name]: store.addValidate('error')})}
                        >错误</Button>
                        <Button 
                            size='small' 
                            style={{ marginLeft: '8px' }} 
                            type='primary' 
                            icon='plus-square'
                            onClick={() => onChange({[name]: store.addValidate('warning')})}
                        >提醒</Button>
                    </div>
                </div>
                <div>
                {
                    store.validates.map((item, i) => (
                        <ValidatorItem 
                            item={ item } 
                            key={ i } 
                            index={ i }
                            onChange={(key, value) => {
                                onChange({
                                    [name]: store.changeValidates(i, key, value)
                                })
                            }}
                            onDelete={() => {
                                onChange({
                                    [name]: store.deleteValidates(i)
                                })
                            }} 
                        />
                    ))
                }
                </div>
            </div>
        );
    };
};

class ValidatorItem extends React.Component{
    constructor(prosp){
        super(prosp);
        this.state = {
            show: false
        };
    }
    render(){
        const { item, onChange, onDelete } = this.props;
        return (
            <div>
                <Alert
                    style={{ marginBottom: '8px' }} 
                    message={(
                        <p style={{position: 'relative', minHeight: "16px" }}>
                            <span >{item.message}</span>
                            <a 
                                style={{display:'block', position: 'absolute', width: "16px",textAlign:'center', right: "-12px", top: 0}} 
                                onClick={ () => this.setState({ show: !this.state.show }) }
                            ><Icon type={ this.state.show ? 'check' : 'edit' } /></a>
                            <a 
                                style={{display:'block', position: 'absolute', width: "16px",textAlign:'center', right: "-32px", top: 0, color: '#f04134'}} 
                                onClick={() => onDelete()}
                            ><Icon type='delete' /></a>
                        </p>
                    )} 
                    type={ item.type } 
                />
                {
                    this.state.show
                    ?   <div style={{ padding: '8px', marginBottom: '8px', backgroundColor: '#f5f5f5', borderRadius: '2px' }}>
                            <RadioGroup value={ item.type } onChange={e => onChange('type', e.target.value)}>
                                <Radio value='error'>错误</Radio>
                                <Radio value='warning'>提醒</Radio>
                            </RadioGroup>
                            <Input 
                                style={{ margin: '4px 0' }} 
                                value={ item.message } 
                                placeholder="错误提示"
                                onChange={e => onChange('message', e.target.value)}
                            />
                            <Input 
                                value={ item.value } 
                                placeholder='表达式' 
                                type='textarea'
                                onChange={e => onChange('value', e.target.value)} 
                            />
                        </div>
                    : void 0
                }
            </div>
        );
    }
};

export default FieldSettingView;

export const CodeView = (props) => (
    <NumberType label="编号" {...props} name='code' />
);

export const TitleView = (props) => (
    <InputType label='标题' {...props} name='label' />
);

export const PlaceHolderView = (props) => (
    <InputType label='提示' {...props} name='placeholder' />
);

export const ShowLabelView = (props) => (
    <SwitchType label="显示标题" {...props} name='showLabel' />
);

export const InlineView = (props) => (
    <SwitchType label="水平排列" {...props} name='inline' />
);

export const RequiredView = (props) => (
    <SwitchType label="必填" {...props} name='required' />
)

export const DescriptionView = (props) => (
    <TextAreaType label="备注" {...props} name='field_options.description' />
);

export const OptionsInlineView = props => (
    <RadioType 
        label='选项排列' 
        {...props} 
        name='field_options.inline' 
        options={[{value: 'true', label: '水平'},{value: 'false', label: '垂直'}]} 
    />
);

export const SetOptionsView = props => (
    <SetOptionsType label='设置选项' {...props} name='field_options.options' />
);

export const ValidateView = props => (
    <CodeType label='逻辑验证' {...props} name='validates' />
);