import * as React from 'react';
import { inject, observer } from 'mobx-react';
import {
    Input,
    Radio,
    Popover,
    Mention    
} from 'antd';
const RadioGroup = Radio.Group;
const { toString } = Mention;

//import {Editor, EditorState, convertToRaw, convertFromRaw} from 'draft-js';

const FieldView = observer(class extends React.Component {
    devView( store ){
        const { 
            settings,
            setting_title,
            setting_selected,
            changeSettingSelect 
        } = store;
        const _title=(
            <div style={{ display: 'table', width: '100%' }} >
                <div
                    style={{
                        display: 'table-cell', 
                        width: `${26 * setting_title.length + 4}px`, 
                        overflow: 'hidden',
                        verticalAlign: 'top',
                        paddingRight: '4px'
                    }}
                >
                {
                    setting_title.map(item => (
                        <div 
                            style={{ 
                                float: 'left', 
                                width: '26px', 
                                textAlign:"center",
                                padding: '4px 0',
                            }} 
                            key={ item.type }
                        >
                            <a
                                style={{
                                    display: 'block',
                                    width: '26px',
                                    margin: '0 auto',
                                    color: '#666',
                                    borderRadius: '2px',
                                    lineHeight: '24px', 
                                    backgroundColor:  item.type === setting_selected ? 'rgba(150, 150, 150, .2)' : 'transparent'
                                }}
                                onClick={() => changeSettingSelect(item.type)}
                            >
                                <i className={ item.icon } style={{ fontSize: '14px'}} ></i>
                            </a>
                        </div>
                    ))
                }
                </div>
                <div
                    style={{ 
                        display: 'table-cell', 
                    }}
                >
                    <p>
                    {
                        setting_title.filter(item => item.type === setting_selected)[0].label
                    }
                    </p>
                </div>
            </div>
        )
        const _content = (
            <div style={{width: '240px', height: '180px', overflowY: 'scroll', marginRight: '-16px', paddingRight: '16px'}} >
            {
                settings[setting_selected].map((View, i) => (
                    <View key={ i } store={ store } />
                ))
            }
            </div>
        );
        return (
            <div 
                style={{
                    padding: '8px 4px'
                }}
            >
                <Popover
                    title={ _title }
                    content={ _content }
                    placement='right'
                    arrowPointAtCenter={ true }
                    trigger='click'
                >
                    <a style={{ fontSize: "13px" }}>[ 编辑 ]</a>
                </Popover>
            </div>
        );
    };
    render(){
        const { store, SubView, disabled, mode } = this.props;
        const {
            label,
            showLabel,
            cid,
            code,
            required,
            inline,
            getOptions
        } = store;
        return (
            <div
                style={{ padding: '5px' }}
            >
                <div
                    className={`crf-field-content ${ inline ? 'inline' : '' }`}
                >
                {
                    showLabel
                    ?   <div className="field-label" >
                            <label htmlFor={`#${cid}`}>
                            {
                                required
                                ?   <span style={{ color: '#ff2500' }} > * </span>
                                :   void 0
                            }
                            { code } - { label }
                            </label>
                        </div>
                    :   void 0
                }
                    <div className='field-content' style={ showLabel ? void 0 : { paddingLeft: 0 }} >
                        <SubView store={ store } disabled={ disabled } onChange={ (value) => this.props.onChange(value) } />
                    </div>
                </div>
                {
                    getOptions('field_options.description')
                    ?   <div
                            style={{ 
                                borderBottom: '1px dashed #ccc', 
                                borderTop: '1px dashed #ccc',
                                marginTop: '12px',
                                color: '#999',
                                fontSize: '14px',
                                lineHeight: '28px',
                                fontStyle: 'italic' 
                            }}
                        >{ getOptions('field_options.description') }</div>
                    : void 0   
                }
                {
                    // mode === 'DEV'
                    // ?   this.devView( store )
                    // :   void 0    
                }
            </div>
        );
    };
});

export class TextView extends React.Component{
    render(){
        const {
            value,
            placeholder,
            cid,
        } = this.props.store;
        const { disabled, onChange } = this.props;
        return (
            <div>
                <Input 
                    disabled={ disabled || this.props.store.disabled }
                    id={ cid }
                    value={ disabled ? "" : value }
                    placeholder={ placeholder }
                    onChange={ e => onChange({value: e.target.value}) }
                />
            </div>  
        );
    };
};

export class RadioView extends React.Component{
    render(){
        const {
            value,
            field_options
        } = this.props.store;
        const { disabled, onChange } = this.props;
        const { options, inline } = field_options
        const _style={
            display: 'block',
            height: '28px',
            lineHeight: '28px',
        }
        return (
            <div style={{lineHeight: "28px"}}>
                <RadioGroup
                    value={ disabled ? "" : value }
                    onChange={ e => onChange({value: e.target.value}) } 
                >
                {
                    options.map(item => (
                        <Radio
                            disabled={ disabled || this.props.store.disabled } 
                            value={ item.code }
                            style={ inline ? void 0 : _style } 
                            key={ item.code } 
                        >
                        { item.label }
                        </Radio>
                    ))
                }
                </RadioGroup>
            </div>  
        );
    };
};

export class AutoCompleteView extends React.Component{
    render(){
        const {
            onSearchChange,
            loading,
            searchCodes,
        } = this.props.store;
        const { disabled } = this.props;
        return (
            <div>
                <Mention
                    style={{ width: '100%', height: 100 }}
                    onChange={ editorState => {
                        //console.log(toString(editorState));
                    }}
                    disabled={ disabled || this.props.store.disabled }
                    onSearchChange = { value => onSearchChange(value) }
                    loading = { loading }
                    suggestions={ searchCodes.map(item => item.name) }
                    multiLines
                />
            </div>
        );
    };
};

export default FieldView;

//  <Editor
// //handlePastedText = { (text, html) => console.log(text) }
//     //blockRendererFn={ block => console.log(block.getText()) }
//     editorState={this.state.editorState} 
//     onChange={this.onChange} 
// />    