import * as React from 'react';
import { inject, observer } from 'mobx-react';
import SideBar from './SideBar';
import './app.scss';
import {
    Switch,
    Route
} from 'react-router-dom';
import FormEdit from './FormEdit';
import FormMainView from './FormMainView';
import {mapType} from './FieldModel';
import FieldView from './FieldView';
import RelationView from './RelationView';

export const FormBuilder = inject('crf', 'crfFormEdit', 'api')(observer(class extends React.Component{
    componentWillMount(){
        const { formsApi } = this.props
        this.props.crf.getForms( this.props.api[formsApi] );
    };
    render(){
        const { 
            forms,
            formsLoading
        } = this.props.crf
        const {
            match,
            location
        } = this.props;
        const {
            isDrag,
            dragTop,
            dragLeft,
            dragHeight,
            dragWidth,
            onMove,
            stopDrag,
            dragFieldType
        } = this.props.crfFormEdit;
        return (
            <div
                className='crf-warpper'
                onMouseMove={e => onMove(e)}
                onMouseUp={e => stopDrag(e)}
            >
                <SideBar
                    loading = { formsLoading }
                    forms = { forms }
                    path={ match.url }
                    url={ location.pathname } 
                />
                <div 
                    className='crf-container' 
                >
                    
                    <Route path={ `${match.url}` } exact component={ FormMainView } />
                    <Route path={ `${match.url}/:id` } exact component={ FormMainView } />
                </div>
                <FormEdit />
                <RelationView />
                {
                    isDrag
                    ?   <div
                            style={{ 
                                position: 'absolute', 
                                top: dragTop, 
                                left: dragLeft, 
                                width: `${ dragWidth }px`,
                                minHeight: `${ dragHeight }px`,
                                boxSizing: 'border-box',
                                backgroundColor: '#F4F4F4',
                                boxShadow: '2px 2px 8px rgba(160, 160, 160, .2)',
                                border: '1px solid #EEE'
                            }}
                        >
                            <FieldView 
                                store={ mapType.getStore(dragFieldType) } 
                                SubView={ mapType.getView( dragFieldType ) } 
                                disabled={true} 
                            />
                        </div>
                    :   void 0   
                }
            </div>
        );
    };
}));