import * as React from 'react';
import { inject, observer } from 'mobx-react';
import { 
    Tooltip, 
    Icon,
    Input 
} from 'antd';

const { Search } = Input;

const FormEdit = inject('crfFormEdit')(observer(class extends React.Component{
    componentWillMount(){
        this.props.crfFormEdit.setFields()
    };
    render(){
        const { 
            tabs, 
            activeTab,
            ActiveView,
            changeTab,
            fieldTypes,
            startDrag,
        } = this.props.crfFormEdit;
        return (
            <div className='crf-edit-view-content' >
                <div
                    className='edit-view-header'
                >
                    <ul>
                    {
                        tabs.map((item, i) => (
                            <li key={i} >
                                <Tooltip
                                    placement='bottom'
                                    title={ item.name }
                                >
                                    <a
                                        className={ item.type === activeTab ? 'active' : void 0 }
                                        onClick={() => {changeTab(item.type, item.view)} }
                                    >
                                        <Icon type={ item.icon } />
                                    </a>
                                </Tooltip>
                            </li>
                        ))
                    }
                    </ul>
                </div>
                <div className="edit-view-content" >
                    <div
                        style={{ width: '100%', height: '100%', overflow: 'auto' }}
                    >
                        <ActiveView />
                    </div>
                </div>
                <div className='edit-view-fields' >
                    <div
                        className='edit-fields-header'
                    >
                        <h4>问题类型</h4>
                    </div>
                    <div className='edit-fields-content' >
                        <div
                            style={{ width: '100%', height: '100%', overflow: 'auto' }}
                        >
                            <ul>
                            {
                                fieldTypes.map((item, i) => (
                                    <FieldTypeItem 
                                        {...item}
                                        onDrag={ startDrag }
                                        key={ i }
                                        index = { i }
                                    />
                                ))
                            }
                            </ul>
                        </div>
                    </div>
                </div>
                <div className='edit-view-footer' >
                    <Search
                        placeholder='请输入问题类型'
                    />
                </div>
            </div>
        );
    };
})); 

class FieldTypeItem extends React.Component {
    render(){
        const {icon, title, type, note, onDrag, draging, index} = this.props;
        return (
             <li
                ref={ _type => this._type = _type }  
                style={ draging ? { opacity: .4 } : void 0 }
             >
                <div className='field-content'>
                    <div className='field-icon'>
                        <span>
                            <i className={ icon }></i>
                        </span>
                    </div>
                    <div className='field-title' >
                        <p>
                            <span>{ title }</span> - 
                            { note }
                        </p>
                    </div>
                </div>
                <div
                    style={{
                        position: 'absolute',
                        width: '100%',
                        height: '100%',
                        top: 0,
                        left: 0,
                        cursor: 'pointer',
                        backgroundColor: 'rgba(0, 0, 0, 0)'
                    }}
                    onMouseDown={(e) => {
                        onDrag(index, type, e, this._type)
                    }}
                ></div>
            </li>
        );
    };
};

export default FormEdit;