import { action,  extendObservable} from 'mobx';
import { 
    AddFieldView,
    LayoutsView,
    BaseView,
    MateFieldsView,
    MateView
} from './EditViews';
import { isEmpty , copy} from '../utils'
import crfDesign from './FieldDesignModel';

// {
//     name: '添加问题',
//     type: 'addField',
//     icon: 'tool',
//     view: AddFieldView
// },
export const field_types = {
    TEXT: 'text',
    AUTOCOMPLETE: 'autocomplete',
    CHECKBOX: 'checkbox',
    RADIO: 'radio',
    DATE: 'date',
    DROPDOWN: 'dropdown',
    LABEL: 'label'
};

class FormEditModel {
    constructor(){
        extendObservable(this, {
            activeTab: 'layouts',
            isDrag: false,
            dragTop: 0,
            dragLeft: 0,
            dragWidth: 0,
            dragHeight: 0,
            dragIndex: 0,
            dragFieldType: '',
            _move: {
                x: 0,
                y: 0
            },
            ActiveView: LayoutsView,
            tabs: [{
                name: '排版布局',
                type: 'layouts',
                icon: 'solution',
                view: LayoutsView
            },{
                name: '基本信息',
                type: 'base',
                icon: 'edit',
                view: BaseView
            },{
                name: '数据集',
                type: 'mateFields',
                icon: 'book',
                view: MateFieldsView
            },{
                name: '数据元',
                type: 'mate',
                icon: 'appstore-o',
                view: MateView
            }],
            fieldTypes: [{
                type: field_types.TEXT,
                title: '单行文本',
                note: '单行文本用于单行文字量较少的文本填空',
                icon: 'icon-typography'
            },{
                type: field_types.AUTOCOMPLETE,
                title: "自动匹配文本",
                note: '自动匹配文本中的关键字',
                icon: 'icon-pen2' 
            },{
                type: field_types.RADIO,
                title: '单选题',
                note: '用于选取唯一答案(建议选项较少时使用)',
                icon: 'icon-circles'
            },{
                type: field_types.CHECKBOX,
                title: '多选题',
                note: '用于选择多个答案',
                icon: 'icon-checkbox-partial2'
            },{
                type: field_types.DATE,
                title: '日期选择',
                note: '用于时间日期的选取',
                icon: 'icon-calendar2'
            }],
            setFields: action(fieldTypes => {
                let _fieldTypes = copy( this.fieldTypes );
                if( !isEmpty(fieldTypes) ){
                    _fieldTypes = fieldTypes;
                };
                for( let i = 0; i < _fieldTypes.length; i++ ){
                    _fieldTypes[i].draging = false;
                };
                this.fieldTypes = _fieldTypes;
            }),
            changeTab: action((type, view) => {
                this.activeTab = type;
                this.ActiveView = view;
            }),
            startDrag: action(( index, type, e, node ) => {
                let _pos = {
                    x: 0,
                    y: 0
                };
                this.getOffeset(_pos, node);
                this.dragWidth = node.offsetWidth;
                this.dragHeight = node.offsetHeight;
                this._move.x = e.clientX;
                this._move.y = e.clientY;
                this.dragLeft = _pos.x;
                this.dragTop = _pos.y;
                this.dragFieldType = type;
                this.changeDraging(index, true);
                this.dragIndex = index;
            }),
            changeDraging: action((index, type) => {
                this.isDrag = type;
                const _filedTypes = copy( this.fieldTypes );
                _filedTypes[index].draging = type;
                this.fieldTypes = _filedTypes;
            }),
            stopDrag: action(e => {
                if( this.isDrag ){
                    crfDesign.addField(this.dragFieldType);
                    this.changeDraging(this.dragIndex, false);
                    crfDesign.setElementInitCancel();
                };
            }),
            onMove: action(e => {
                if( this.isDrag ){
                    const _move = {
                        x: e.clientX - this._move.x,
                        y: e.clientY - this._move.y
                    };
                    this._move = {
                        x: e.clientX,
                        y: e.clientY
                    };
                    this.dragLeft = this.dragLeft + _move.x;
                    this.dragTop = this.dragTop + _move.y;
                    crfDesign.changeElementInit(this.dragLeft, this.dragTop);
                };
            }),
        });
    };
    getOffeset(_pos, node){
        _pos.x = _pos.x + node.offsetLeft;
        _pos.y = _pos.y + node.offsetTop;
        if( node.offsetParent ){
            this.getOffeset(_pos, node.offsetParent);
        };
    };
};

const crfFormEdit = new FormEditModel();

export default crfFormEdit;