import * as React from 'react';
import { inject, observer } from 'mobx-react';
import { Icon, Button, Modal } from 'antd';
import FieldContent from './FieldContent';

const FormMainView = inject('crfFormMain', 'crf', 'crfDesign')(observer(class extends React.Component {
    componentDidMount(){
        this.props.crfDesign.setMainElement(this._main);
}
    render(){
        const { 
            form,
            page,
            zoom,
            plusZoom,
            minuZoom,
            showPreview,
            openPreview,
            closePreview,
            previewKey
        } = this.props.crfFormMain;
        return (
            <div style={{ position: 'absolute', width: '100%', height: '100%' }} >
                <div className='main-view-header' >
                    <h4>
                        <span>
                            <Icon type="file-text" />
                        </span>
                        { form.name }
                    </h4>
                    <div style={{display: 'table-cell', width: '50px'}}>
                        <Button 
                            size='small' 
                            type='primary'
                            onClick={ () => openPreview() } 
                        >预览</Button>
                    </div>
                </div>
                <div className='main-view-content' >
                    <div
                        ref={ main => this._main = main }
                        style={{ width: '100%', height: '100%',overflow: 'scroll' }}
                    >
                        <div style={{ width: page.width, padding: '24px 0', margin: '0 auto' }} >
                            <div className='view-page' style={{ width: page.width }} >
                                <div style={{ minHeight: page.height }}>
                                    <FieldContent
                                        mode='DEV'
                                        width={ page.width - 48 } 
                                        name={ form.name }
                                    />
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
                <div className='main-view-footer' >
                    <p>宽高：{`${ page.width } x ${ page.height }`}</p>
                    <div className='footer-btn-content' >
                        <div className='footer-btn-box' >
                            <a
                                onClick={() => plusZoom()}
                            >
                                <i className='icon-plus3' ></i>
                            </a>
                            <span>{`${parseInt(zoom * 10)}%`}</span>
                            <a
                                onClick={() => minuZoom()}
                            >
                                <i className='icon-minus3' ></i>
                            </a>
                        </div>
                    </div>
                </div>
                <Modal
                    key={`preview_${previewKey}`}
                    visible={ showPreview }
                    onCancel={ () => closePreview() }
                    title='表单预览'
                    width="900px"
                    style={{padding: 0, top: "10px"}}
                    footer={null}
                >
                    <div
                        style={{ margin: '0 -16px',height: window.document.body.clientHeight - 48 - 32 - 20 + 'px', overflow: 'auto' }}
                    >
                        <FieldContent
                            mode='PREVIEW'
                            width={ page.width - 48 } 
                            name={ form.name }
                        />
                    </div>
                </Modal>
            </div>
        );
    };
}));

export default FormMainView