import BaseModel from './BaseModel';
import FormModel from './form/FormModel';
import { inject, observer, Provider } from 'mobx-react';
import { isEmpty, isString } from './utils';
import * as React from 'react';
import { render } from 'react-dom';
import Api from './Api';
import crf from './CRFForm/FormModel';
import crfFormEdit from './CRFForm/FormEditModel';
import crfFormMain from './CRFForm/FormMainModel';
import crfDesign from './CRFForm/FieldDesignModel';
import crfRelation from './CRFForm/RelationModel';
import { Form, Modal,Spin } from 'antd';

const App = (function(){
    let _models = { 
        crf,
        crfFormEdit,
        crfFormMain,
        crfDesign,
        crfRelation
     };
    return class {
        Router;
        http;
        model( _config, type='default' ){
            try{
                const { namespace } = _config;
                if( isEmpty( namespace ) ){
                    throw new Error('请为model输入正确的命名');
                };
                switch(type){
                    case 'default':
                        _models[namespace] = new BaseModel( _config );
                        break;
                    case 'form' :
                        _models[namespace] = new FormModel( _config );
                        break;
                    default:
                        _models[namespace] = new BaseModel( _config );
                        break;
                };
                
            }catch(error){
                throw new Error( error );
            };
        };

        api(config){
            this.http = new Api(config);
        };

        view(...injectName){
            if( isEmpty( injectName ) ){
                throw new Error('view方法参数不能为空');
            };
            if( !isString(injectName[0]) ){
                return injectName[0];
            };
            return function(render){
                return inject(...injectName)(observer(render));
            };
        };

        formview(formModel='', injectName=[]){
            injectName.push(formModel)
            return (Render => {
                return Form.create({
                    onValuesChange(props, values){
                        _models[formModel]['changeValue'](values)
                    }
                })(this.view(...injectName)(props => (
                    <Form
                        layout={ props.layout }
                        hideRequiredMark={ props.hideRequiredMark }
                        style = {Object.assign({}, {padding: '30px 0'}, props.style)}
                        onSubmit = { e => _models[formModel].onSubmit(e, props.form) }
                    >
                        <Render {...props} />
                        <Modal
                            footer = { null }
                            closable = { false }
                            visible = { props[formModel]['showSubmitting'] && props[formModel]['submitting'] }
                        >
                            <div style={{padding: '20px 12px'}}>
                                <Spin size='small' />
                                <span style={{ marginLeft: '12px', fontSize: '14px' }}>表单提交中,请不要关闭窗口!</span>
                            </div>
                        </Modal>
                    </Form>
                )));
            });
        };

        router( Router ){
            this.Router = (
                <Provider {..._models} api={ this.http }>
                    <Router />
                </Provider>
            ); 
        };

        run(node){
            const { Router } = this;
            render(Router, document.getElementById(node));
        };
    }

})();

const app = new App()

export default app

