import { observable, action, extendObservable } from 'mobx';
import BaseModel from '../BaseModel';
import app from '../app';

class FormModel extends BaseModel {
    constructor(_configs){
        _configs['formData'] = {};
        _configs['submitting'] = false;
        if( !('showSubmitting' in _configs) ){
            _configs['showSubmitting'] = true;
        };
        let _actions = { };
        if( 'actions' in _configs ){
            _actions = _configs['actions']
        };
        _actions['onSubmit'] = ( event, form ) => {
            event.preventDefault();
            form.validateFields(async (err, values) => {
                if(!err){
                    if(this.beforeSubmit){
                        this.beforeSubmit(values);
                    };
                    this.submitting = true;
                    const resp = await app.http[this.api](values);
                    if( this.afterSubmit && resp ){
                        this.afterSubmit( resp.data );
                    };
                    this.submitting = false;      
                }else{
                    this.localError(err, values)
                };
            })
        };
        _actions['changeValue'] = (values) => {
            this.formData = Object.assign({}, this.formData, values);
        };
        _configs.actions = _actions;
        super( _configs );
    }
};

export default FormModel;