export const isArray = arr => is( arr, "Array" );
export const isObject = obj => is( obj, "Object" );
export const isString = str => is( str, "String" );
export const isNumber = num => is( num, "Number" );
export const isFnc = fnc => is( fnc, "Function");
export const isUndefined = und => is( und, 'Undefined' );
export const isNull = _null => is( _null, 'Null' );

/**
 * @name 判断变量是否为空
 * @param {*} t
 * 如果是undefined或null 返回true
 * 如果是string 判断是否为空字符串
 * 如果是number 判断是否为NaN
 * 如果是Object或Array 判断是否有成员
 * 如果非上述类型 返回false 
 */
export function isEmpty( t ){
    if( isUndefined( t ) ){
        return true;
    };
    if( isNull( t ) ){
        return true;
    }
    if( isString( t ) ){
        return t === '';
    };
    if( isNumber( t ) ){
        return t === NaN;
    };
    if( isObject(t) ){
        if( isArray(t) ){
            return t.length === 0;
        };
        for( let key in t ){
            return false;
        };
        return true;
    };
    return false;
};

export function copy( obj ){
    if( isObject( obj ) ){
        if( !isUndefined(obj.length) ){
            let _arr = [ ];
            for( let i = 0; i < obj.length; i++ ){
                _arr.push(obj[i]);
            };
            return _arr;
        };
        let _obj = { };
        for( let key in obj ){
            _obj[ key ] = obj[key];
        };
        return _obj;
    };
    return obj;
};

export function keys(obj){
    let _keys = [ ];
    if( !isObject(obj) ){
        return _keys;
    };
    for( let key in obj ){
        _keys.push(key);
    };
    return _keys;
};

export function values(obj){
    let _values = [ ];
    if( !isObject(obj) ){
        return _values;
    };
    for( let key in obj ){
        _values.push( obj[key] );
    };
    return _values;
};

export function treeToObj( arr ){
    const _transform = (obj, arr) => {
        for(let i = 0; i < arr.length; i++){

            obj[ arr[i].key ] = { };
            for( let key in arr[i] ){
                if( key !== 'children' ){
                    obj[arr[i].key][key] = arr[i][key];
                };
            };
            if( arr[i].children && arr[i].children.length ){
                _transform(obj, arr[i].children);
            }
        };
    };
    let _obj = { };
    _transform(_obj, arr);
    return _obj;
};

export function getValueByDeepName( obj, name ){
    if(isEmpty(name)){
        return obj;
    };
    const _nameArr = name.split('.');
    if( _nameArr.length > 1 ){
        let _v;
        for( let i = 0; i < _nameArr.length; i++ ){
            _v = i === 0 ? obj[_nameArr[i]] : _v[_nameArr[i]];
        };
        return _v;
    };
    return obj[name];
};

export function setByName(obj, name, value){
    let _obj = copy(obj);
    const _nameArr = name.split('.');
    if( _nameArr.length > 1 ){
        let _f = { };
        for(let i = _nameArr.length - 1; i >= 0; i--){
            let _arr = [];
            for( let j = 0; j < i; j++ ){
                _arr.push(_nameArr[j]);
            };
            let _p = getValueByDeepName(_obj, _arr.join('.'));
            if( _p.length ){
                _p[_nameArr[i]] = _f;
                _f = _p;
                continue; 
            };
            _f = i === _nameArr.length - 1 ? { [_nameArr[i]]: value } : {[_nameArr[i]]: _f};
            _f = Object.assign({}, getValueByDeepName(_obj, _arr.join('.')), _f);    
        };
        _obj = Object.assign({}, _obj, _f);
    }else{
        _obj = Object.assign({}, _obj, {[name]: value});
    };
    return _obj
};

function is( obj, type ){
    return (type === "Null" && obj === null) ||
    (type === "Undefined" && obj === void 0 ) ||
    (type === "Number" && isFinite(obj)) ||
    Object.prototype.toString.call(obj).slice(8,-1) === type;
};